indexing
	description: "Interfaces to C floating-point functions from The GNU MP Bignum Library"
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPF_FUNCTIONS

feature -- Initialization

	mpf_set_default_prec (prec: NATURAL_32) is
			-- Set the default precision to be at least `prec' bits.
			-- All subsequent calls to `mpf_init' will use this precision,
			-- but previously initialized variables are unaffected.
		external
			"C macro signature (unsigned long) use %"gmp.h%""
		end

	mpf_get_default_prec: NATURAL_32 is
			-- Set the default precision to be at least `prec' bits.
			-- All subsequent calls to `mpf_init' will use this precision,
			-- but previously initialized variables are unaffected.
		external
			"C macro signature: unsigned long use %"gmp.h%""
		end

	mpf_init (x: POINTER) is
			-- Initialize `x' to 0.
			-- Normally, a variable should be initialized once only
			-- or at least be cleared, using `mpf_clear', between
			-- initializations.
			-- The precision of `x' is undefined unless a default
			-- precision has already been established by a call to
			-- `mpf_set_default_prec'.
		external
			"C macro signature (mpf_ptr) use %"gmp.h%""
		end

	mpf_init2 (x: POINTER; prec: NATURAL_32) is
			-- Initialize `x' to 0 and set its precision to
			-- be at least `prec' bits.
			-- Normally, a variable should be initialized once
			-- only or at least be cleared, using `mpf_clear',
			-- between initializations.
		external
			"C macro signature (mpf_ptr, unsigned long) use %"gmp.h%""
		end

	mpf_clear (x: POINTER) is
			-- Free the space occupied by `x'.
			-- Make sure to call this function for all
			-- mpf_t variables when you are done with them.
		external
			"C macro signature (mpf_ptr) use %"gmp.h%""
		end

	mpf_get_prec (x: POINTER): NATURAL_32 is
			-- Return the current precision of `op', in bits.
		external
			"C macro signature (mpf_srcptr): unsigned long use %"gmp.h%""
		end

	mpf_set_prec (rop: POINTER; prec: NATURAL_32) is
			-- Set the precision of `rop' to be at least `prec' bits.
			-- The value in `rop' will be truncated to the new precision.
			-- This function requires a call to realloc, and so should
			-- not be used in a tight loop.
		external
			"C macro signature (mpf_ptr, unsigned long) use %"gmp.h%""
		end

	mpf_set_prec_raw (rop: POINTER; prec: NATURAL_32) is
			-- Set the precision of `rop' to be at least `prec' bits,
			-- without changing the memory allocated.
			-- `prec' must be no more than the allocated precision for
			-- `rop', that being the precision when `rop' was
			-- initialized, or in the most recent `mpf_set_prec'.
			-- The value in `rop' is unchanged, and in particular if it
			-- had a higher precision than `prec' it will retain that
			-- higher precision.
			-- New values written to `rop' will use the new `prec'.
			-- Before calling `mpf_clear' or the full `mpf_set_prec',
			-- another `mpf_set_prec_raw' call must be made to restore
			-- `rop' to its original allocated precision.
			-- Failing to do so will have unpredictable results.
			-- `mpf_get_prec' can be used before `mpf_set_prec_raw' to
			-- get the original allocated precision.
			-- After `mpf_set_prec_raw' it reflects the `prec' value set.
			-- `mpf_set_prec_raw' is an efficient way to use an mpf_t
			-- variable at different precisions during a calculation,
			-- perhaps to gradually increase precision in an iteration,
			-- or just to use various different precisions for different
			-- purposes during a calculation.
		external
			"C macro signature (mpf_ptr, unsigned long) use %"gmp.h%""
		end

feature -- Assignment

	mpf_set (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_set_ui (rop: POINTER; op: NATURAL_32) is
			-- Set the value of `rop' from `op'.
		external
			"C macro signature (mpf_ptr, unsigned long) use %"gmp.h%""
		end

	mpf_set_si (rop: POINTER; op: INTEGER_32) is
			-- Set the value of `rop' from `op'.
		external
			"C macro signature (mpf_ptr, signed long) use %"gmp.h%""
		end

	mpf_set_d (rop: POINTER; op: REAL_64) is
			-- Set the value of `rop' from `op'.
		external
			"C macro signature (mpf_ptr, double) use %"gmp.h%""
		end

	mpf_set_z (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C macro signature (mpf_ptr, mpz_srcptr) use %"gmp.h%""
		end

	mpf_set_q (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
		external
			"C macro signature (mpf_ptr, mpq_srcptr) use %"gmp.h%""
		end

	mpf_set_str (rop, str: POINTER; base: INTEGER_32): INTEGER_32 is
			-- Set the value of `rop' from the string in `str'.
			-- The string is of the form `M@N' or, if the base is 10 or less, alternatively `MeN'.
			-- `M' is the mantissa and `N' is the exponent.
			-- The mantissa is always in the specified base.
			-- The exponent is either in the specified base or, if base is negative, in decimal.
			-- The decimal point expected is taken from the current locale, on systems providing
			-- localeconv.
			-- The argument `base' may be in the ranges 2 to 62, or 62 to 2.
			-- Negative values are used to specify that the exponent is in decimal.
			-- For bases up to 36, case is ignored; upper-case and lower-case letters have the same
			-- value; for bases 37 to 62, upper-case letter represent the usual 10..35 while
			-- lower-case letter represent 36..61.
			-- Unlike the corresponding mpz function, the base will not be determined from the
			-- leading characters of the string if base is 0.
			-- This is so that numbers like `0.23' are not interpreted as octal.
			-- White space is allowed in the string, and is simply ignored.
			-- [This is not really true; white-space is ignored in the beginning of the string and
			-- within the mantissa, but not in other places, such as after a minus sign or in the
			-- exponent.
			-- This function returns 0 if the entire string is a valid number in base `base'.
			-- Otherwise it returns 1.
		external
			"C macro signature (mpf_ptr, char *, int): int use %"gmp.h%""
		end

	mpf_swap (rop1, rop2: POINTER) is
			-- Swap `rop1' and `rop2' efficiently.
			-- Both the values and the precisions of
			-- the two variables are swapped.
		external
			"C macro signature (mpf_ptr, mpf_ptr) use %"gmp.h%""
		end

feature -- Combined initialization and assignment

	mpf_init_set (rop, op: POINTER) is
			-- Set the value of `rop' from `op'.
			-- The precision of `rop' will be taken from the active
			-- default precision, as set by `mpf_set_default_prec'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_init_set_ui (rop: POINTER; op: NATURAL_32) is
			-- Set the value of `rop' from `op'.
			-- The precision of `rop' will be taken from the active
			-- default precision, as set by `mpf_set_default_prec'.
		external
			"C macro signature (mpf_ptr, unsigned long) use %"gmp.h%""
		end

	mpf_init_set_si (rop: POINTER; op: INTEGER_32) is
			-- Set the value of `rop' from `op'.
			-- The precision of `rop' will be taken from the active
			-- default precision, as set by `mpf_set_default_prec'.
		external
			"C macro signature (mpf_ptr, signed long) use %"gmp.h%""
		end

	mpf_init_set_d (rop: POINTER; op: REAL_64) is
			-- Set the value of `rop' from `op'.
			-- The precision of `rop' will be taken from the active
			-- default precision, as set by `mpf_set_default_prec'.
		external
			"C macro signature (mpf_ptr, double) use %"gmp.h%""
		end

	mpf_init_set_str (rop, str: POINTER; base: INTEGER_32): INTEGER_32 is
			-- Initialize `rop' and set its value from the string in `str'.
			-- Note that `rop' is initialized even if an error occurs.
			-- (I.e., you have to call `mpf_clear' for it.)
			-- See `mpf_set_str' for details on the assignment operation.
			-- The precision of `rop' will be taken from the active
			-- default precision, as set by `mpf_set_default_prec'.
		external
			"C macro signature (mpf_ptr, char *, int): int use %"gmp.h%""
		end

feature -- Conversion

	mpf_get_d (op: POINTER): REAL_64 is
			-- Convert `op' to a double, truncating
			-- if necessary (ie. rounding towards zero).
			-- If the exponent in `op' is too big or too
			-- small to fit a double then the result is
			-- system dependent.
			-- For too big an infinity is returned when
			-- available.
			-- For too small 0.0 is normally returned.
			-- Hardware overflow, underflow and denorm
			-- traps may or may not occur.
		external
			"C macro signature (mpf_srcptr): double use %"gmp.h%""
		end

	mpf_get_d_2exp (exp: TYPED_POINTER [INTEGER_32]; op: POINTER): REAL_64 is
			-- Convert `op' to a double, truncating if necessary
			-- (ie. rounding towards zero), and with an exponent
			-- returned separately.
			-- The return value is in the range `0.5<=abs(d)<1'
			-- and the exponent is stored to `*exp'.
			-- `d * 2^exp' is the (truncated) `op' value.
			-- If `op' is zero, the return is 0.0 and 0 is stored
			-- to `*exp'.
			-- This is similar to the standard C frexp function.
		external
			"C macro signature (signed long *, mpf_srcptr): double use %"gmp.h%""
		end

	mpf_get_si (op: POINTER): INTEGER_32 is
			-- Convert `op' to a long,
			-- truncating any fraction part.
			-- If `op' is too big for the return type,
			-- the result is undefined.
			-- See also `mpf_fits_slong_p'
			-- (see Miscellaneous).
		external
			"C macro signature (mpf_srcptr): long use %"gmp.h%""
		end

	mpf_get_ui (op: POINTER): NATURAL_32 is
			-- Convert `op' to an unsigned long,
			-- truncating any fraction part.
			-- If `op' is too big for the return type,
			-- the result is undefined.
			-- See also `mpf_fits_ulong_p'
			-- (see Miscellaneous).
		external
			"C macro signature (mpf_srcptr): unsigned long use %"gmp.h%""
		end

	mpf_get_str (str: POINTER; expptr: TYPED_POINTER [INTEGER_32]; base: INTEGER_32; n_digits: NATURAL_64; op: POINTER): POINTER is
			-- Convert `op' to a string of digits in base `base'.
			-- `base' can be 2 to 36.
			-- Up to `n_digits' digits will be generated.
			-- Trailing zeros are not returned.
			-- No more digits than can be accurately represented by op are ever generated.
			-- If `n_digits' is 0 then that accurate maximum number of digits are generated.
			-- If `str' is NULL, the result string is allocated using the current allocation
			-- function.
			-- The block will be `strlen(str)+1' bytes, that being exactly enough for the
			-- string and null-terminator.
			-- If `str' is not NULL, it should point to a block of `n_digits + 2' bytes, that
			-- being enough for the mantissa, a possible minus sign, and a null-terminator.
			-- When `n_digits' is 0 to get all significant digits, an application won't be able
			-- to know the space required, and `str' should be NULL in that case.
			-- The generated string is a fraction, with an implicit radix point immediately to
			-- the left of the first digit.
			-- The applicable exponent is written through `expptr'.
			-- For example, the number 3.1416 would be returned as string "31416" and exponent 1.
			-- When `op' is zero, an empty string is produced and the exponent returned is 0.
			-- A pointer to the result string is returned, being either the allocated block or
			-- the given str.
		external
			"C macro signature (char *, mp_exp_t *, int, size_t, mpf_srcptr): char * use %"gmp.h%""
		end

feature -- Arithmetic

	mpf_add (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' + `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_add_ui (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to `op1' + `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, unsigned long) use %"gmp.h%""
		end

	mpf_sub (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' - `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_ui_sub (rop: POINTER; op1: NATURAL_32; op2: POINTER) is
			-- Set `rop' to `op1' - `op2'.
		external
			"C macro signature (mpf_ptr, unsigned long, mpf_srcptr) use %"gmp.h%""
		end

	mpf_sub_ui (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to `op1' - `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, unsigned long) use %"gmp.h%""
		end

	mpf_mul (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' times `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_mul_ui (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to `op1' times `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, unsigned long) use %"gmp.h%""
		end

	mpf_div (rop, op1, op2: POINTER) is
			-- Set `rop' to `op1' / `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_ui_div (rop: POINTER; op1: NATURAL_32; op2: POINTER) is
			-- Set `rop' to `op1' / `op2'.
		external
			"C macro signature (mpf_ptr, unsigned long, mpf_srcptr) use %"gmp.h%""
		end

	mpf_div_ui (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to `op1' / `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, unsigned long) use %"gmp.h%""
		end

	mpf_sqrt (rop, op: POINTER) is
			-- Set `rop' to the square root of `op'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_sqrt_ui (rop: POINTER; op: NATURAL_32) is
			-- Set `rop' to the square root of `op'.
		external
			"C macro signature (mpf_ptr, unsigned long) use %"gmp.h%""
		end

	mpf_pow_ui (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to op1 raised to the power `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, unsigned long) use %"gmp.h%""
		end

	mpf_neg (rop, op: POINTER) is
			-- Set `rop' to -`op'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_abs (rop, op: POINTER) is
			-- Set `rop' to the absolute value of `op'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_mul_2exp (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to `op1' times 2 raised to `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, unsigned long) use %"gmp.h%""
		end

	mpf_div_2exp (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to op1 divided by 2 raised to `op2'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, unsigned long) use %"gmp.h%""
		end

feature -- Comparison

	mpf_cmp (op1, op2: POINTER): INTEGER_32 is
			-- Compare `op1' and `op2'.
			-- Return a positive value if `op1 > op2',
			-- zero if `op1 = op2', and a negative
			-- value if `op1 < op2'.
		external
			"C macro signature (mpf_srcptr, mpf_srcptr): int use %"gmp.h%""
		end

	mpf_cmp_d (op1: POINTER; op2: REAL_64): INTEGER_32 is
			-- Compare `op1' and `op2'.
			-- Return a positive value if `op1 > op2',
			-- zero if `op1 = op2', and a negative
			-- value if `op1 < op2'.
			-- Ccan be called with an infinity, but
			-- results are undefined for a NaN.
		external
			"C macro signature (mpf_srcptr, double): int use %"gmp.h%""
		end

	mpf_cmp_ui (op1: POINTER; op2: NATURAL_32): INTEGER_32 is
			-- Compare `op1' and `op2'.
			-- Return a positive value if `op1 > op2',
			-- zero if `op1 = op2', and a negative
			-- value if `op1 < op2'.
		external
			"C macro signature (mpf_srcptr, unsigned long): int use %"gmp.h%""
		end

	mpf_cmp_si (op1: POINTER; op2: INTEGER_32): INTEGER_32 is
			-- Compare `op1' and `op2'.
			-- Return a positive value if `op1 > op2',
			-- zero if `op1 = op2', and a negative
			-- value if `op1 < op2'.
		external
			"C macro signature (mpf_srcptr, signed long): int use %"gmp.h%""
		end

	mpf_eq (op1, op2: POINTER; op3: NATURAL_32): INTEGER_32 is
			-- Return non-zero if the first `op3' bits of `op1' and `op2'
			-- are equal, zero otherwise. I.e., test if `op1' and `op2'
			-- are approximately equal.
			-- Caution: Currently only whole limbs are compared, and only
			-- in an exact fashion.
			-- In the future values like 1000 and 0111 may be considered
			-- the same to 3 bits (on the basis that their difference is
			-- that small).
		external
			"C macro signature (mpf_srcptr, mpf_srcptr, unsigned long): int use %"gmp.h%""
		end

	mpf_reldiff (rop, op1, op2: POINTER) is
			-- Compute the relative difference between `op1'
			-- and `op2' and store the result in `rop'.
			-- This is `abs(op1-op2)/op1'.
		external
			"C macro signature (mpf_ptr, mpf_srcptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_sgn (op: POINTER): INTEGER_32 is
			-- Return +1 if `op' > 0,
			-- 0 if `op' = 0, and -1
			-- if `op' < 0.
			-- This function is actually implemented
			-- as a macro.
			-- It evaluates its arguments multiple times.
		external
			"C macro signature (mpf_srcptr): int use %"gmp.h%""
		end

feature -- Input and Output

	mpf_out_str (stream: POINTER; base: INTEGER_32; n_digits: NATURAL_32; op: POINTER): NATURAL_32 is
			--Print `op' to stream, as a string of digits.
			-- Return the number of bytes written, or if an error occurred, return 0.
			-- The mantissa is prefixed with an `0.' and is in the given base, which
			-- may vary from 2 to 36.
			-- An exponent then printed, separated by an `e', or if base is greater
			-- than 10 then by an `@'.
			-- The exponent is always in decimal.
			-- The decimal point follows the current locale, on systems providing localeconv.
			-- Up to `n_digits' will be printed from the mantissa, except that no more digits
			-- than are accurately representable by `op' will be printed.
			-- `n_digits' can be 0 to select that accurate maximum.
		external
			"C macro signature (FILE *, int, size_t, mpf_srcptr): size_t use %"gmp.h%""
		end

	mpf_inp_str (rop, stream: POINTER; base: INTEGER_32): NATURAL_32 is
			-- Read a string in base `base' from `stream', and put the read float in `rop'.
			-- The string is of the form `M@N' or, if the base is 10 or less, alternatively
			-- `MeN'. `M' is the mantissa and `N' is the exponent.
			-- The mantissa is always in the specified base.
			-- The exponent is either in the specified base or, if `base' is negative, in
			-- decimal.
			-- The decimal point expected is taken from the current locale, on systems
			-- providing localeconv.
			-- The argument `base' may be in the ranges 2 to 36, or 36 to 2.
			-- Negative values are used to specify that the exponent is in decimal.
			-- Unlike the corresponding mpz function, the base will not be determined from
			-- the leading characters of the string if `base' is 0.
			-- This is so that numbers like `0.23' are not interpreted as octal.
			-- Return the number of bytes read, or if an error occurred, return 0.
		external
			"C macro signature (mpf_ptr, FILE *, int): size_t use %"gmp.h%""
		end

feature -- Miscellaneous

	mpf_ceil (rop, op: POINTER) is
			-- Set `rop' to `op' rounded to an integer.
			-- Rounds to the next higher integer.
		external
			"C macro signature (mpf_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_floor (rop, op: POINTER) is
			-- Set `rop' to `op' rounded to an integer.
			-- Rounds to the next lower integer.
		external
			"C macro signature (mpf_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_trunc (rop, op: POINTER) is
			-- Set `rop' to `op' rounded to an integer.
			-- Rounds to the next integer towards zero.
		external
			"C macro signature (mpf_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpf_integer_p (op: POINTER): INTEGER_32 is
			-- Return non-zero if `op' is an integer.
		external
			"C macro signature (mpf_srcptr): int use %"gmp.h%""
		end

	mpf_fits_ulong_p (op: POINTER): INTEGER_32 is
			-- Return non-zero if `op' would fit in an
			-- unsigned long data type, when truncated
			-- to an integer.
		external
			"C macro signature (mpf_srcptr): int use %"gmp.h%""
		end

	mpf_fits_slong_p (op: POINTER): INTEGER_32 is
			-- Return non-zero if `op' would fit in a
			-- signed long data type, when truncated
			-- to an integer.
		external
			"C macro signature (mpf_srcptr): int use %"gmp.h%""
		end

	mpf_fits_uint_p (op: POINTER): INTEGER_32 is
			-- Return non-zero if `op' would fit in an
			-- unsigned int data type, when truncated
			-- to an integer.
		external
			"C macro signature (mpf_srcptr): int use %"gmp.h%""
		end

	mpf_fits_sint_p (op: POINTER): INTEGER_32 is
			-- Return non-zero if `op' would fit in a
			-- signed int data type, when truncated
			-- to an integer.
		external
			"C macro signature (mpf_srcptr): int use %"gmp.h%""
		end

	mpf_fits_ushort_p (op: POINTER): INTEGER_32 is
			-- Return non-zero if `op' would fit in an
			-- unsigned short data type, when truncated
			-- to an integer.
		external
			"C macro signature (mpf_srcptr): int use %"gmp.h%""
		end

	mpf_fits_sshort_p (op: POINTER): INTEGER_32 is
			-- Return non-zero if `op' would fit in a
			-- signed short data type, when truncated
			-- to an integer.
		external
			"C macro signature (mpf_srcptr): int use %"gmp.h%""
		end

end
