indexing
	description: "Interfaces to C rational number functions from The GNU MP Bignum Library"
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPQ_FUNCTIONS

feature -- Initialization and assignment

	mpq_init (dest_rational: POINTER) is
			-- Initialize `dest_rational' and set it to 0/1.
			-- Each variable should normally only be initialized
			-- once, or at least cleared out
			-- (using the function `mpq_clear') between each
			-- initialization.
		external
			"C macro signature (mpq_ptr) use %"gmp.h%""
		end

	mpq_clear (rational_number: POINTER) is
			-- Free the space occupied by `rational_number'.
			-- Make sure to call this function for all mpq_t
			-- variables when you are done with them.
		external
			"C macro signature (mpq_ptr) use %"gmp.h%""
		end

	mpq_set (rop, op: POINTER) is
			-- Assign `rop' from `op'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_set_z (rop, op: POINTER) is
			-- Assign `rop' from `op'.
		external
			"C macro signature (mpq_ptr, mpz_srcptr) use %"gmp.h%""
		end

	mpq_set_ui (rop: POINTER; op1, op2: NATURAL_32) is
			-- Set the value of `rop' to `op1/op2'.
			-- Note that if `op1' and `op2' have common factors,
			-- `rop' has to be passed to `mpq_canonicalize' before
			-- any operations are performed on `rop'.
		external
			"C macro signature (mpq_ptr, unsigned long, unsigned long) use %"gmp.h%""
		end

	mpq_set_si (rop: POINTER; op1: INTEGER_32; op2: NATURAL_32) is
			-- Set the value of `rop' to `op1/op2'.
			-- Note that if `op1' and `op2' have common factors,
			-- `rop' has to be passed to `mpq_canonicalize' before
			-- any operations are performed on `rop'.
		external
			"C macro signature (mpq_ptr, signed long, unsigned long) use %"gmp.h%""
		end

	mpq_set_str (rop, str: POINTER; base: INTEGER_32): INTEGER_32 is
			-- Set `rop' from a null-terminated string `str' in the given base.
			-- The string can be an integer like 41 or a fraction like 41/152.
			-- The fraction must be in canonical form, or if not then
			-- `mpq_canonicalize' must be called.
			-- The numerator and optional denominator are parsed the same as in
			-- `mpz_set_str' (see MPZ_FUNCTIONS).
			-- White space is allowed in the string, and is simply ignored.
			-- The base can vary from 2 to 62, or if base is 0 then the leading
			-- characters are used: 0x or 0X for hex, 0b or 0B for binary, 0 for
			-- octal, or decimal otherwise.
			-- Note that this is done separately for the numerator and denominator,
			-- so for instance 0xEF/100 is 239/100, whereas 0xEF/0x100 is 239/256.
			-- The return value is 0 if the entire string is a valid number, or 1
			-- if not.
		external
			"C macro signature (mpq_ptr, char *, int): int use %"gmp.h%""
		end

	mpq_swap (rop1, rop2: POINTER) is
			-- Swap the values `rop1' and `rop2' efficiently.
		external
			"C macro signature (mpq_ptr, mpq_ptr) use %"gmp.h%""
		end

	mpq_canonicalize (op: POINTER) is
			-- Remove any factors that are common
			-- to the numerator and denominator of
			-- `op', and make the denominator positive.
		external
			"C macro signature (mpq_ptr) use %"gmp.h%""
		end

feature -- Conversion

	mpq_get_d (op: POINTER): REAL_64 is
			-- Convert `op' to a double, truncating if necessary (ie. rounding towards zero).
			-- If the exponent from the conversion is too big or too small to fit a double
			-- then the result is system dependent.
			-- For too big an infinity is returned when available.
			-- For too small 0.0 is normally returned.
			-- Hardware overflow, underflow and denorm traps may or may not occur.
		external
			"C macro signature (mpq_srcptr): double use %"gmp.h%""
		end

	mpq_set_d (rop: POINTER; op: REAL_64) is
			-- Set `rop' to the value of `op'.
			-- There is no rounding, this conversion is exact
		external
			"C macro signature (mpq_ptr, double) use %"gmp.h%""
		end

	mpq_set_f (rop, op: POINTER) is
			-- Set `rop' to the value of `op'.
			-- There is no rounding, this conversion is exact
		external
			"C macro signature (mpq_ptr, mpf_srcptr) use %"gmp.h%""
		end

	mpq_get_str (str: POINTER; base: INTEGER_32; op: POINTER): POINTER is
			-- Convert `op' to a string of digits in base `base'.
			-- The base may vary from 2 to 36.
			-- The string will be of the form `num/den', or if
			-- the denominator is 1 then just `num'.
			-- If `str' is NULL, the result string is allocated
			-- using the current allocation function.
			-- The block will be strlen(str)+1 bytes, that being
			-- exactly enough for the string and null-terminator.
			-- If `str' is not NULL, it should point to a block of
			-- storage large enough for the result, that being
			-- mpz_sizeinbase (mpq_numref(op), base) +
			-- mpz_sizeinbase (mpq_denref(op), base) + 3
			-- The three extra bytes are for a possible minus sign,
			-- possible slash, and the null-terminator.
			-- A pointer to the result string is returned, being
			-- either the allocated block, or the given `str'.
		external
			"C macro signature (char *, int, mpq_srcptr): char * use %"gmp.h%""
		end

feature -- Arithmetic

	mpq_add (sum, addend1, addend2: POINTER) is
			-- Set `sum' to `addend1' + `addend2'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_sub (difference, minuend, subtrahend: POINTER) is
			-- Set `difference' to `minuend' - `subtrahend'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_mul (product, multiplier, multiplicand: POINTER) is
			-- Set `product' to `multiplier' times `multiplicand'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_mul_2exp (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to `op1' times 2 raised to `op2'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr, unsigned long) use %"gmp.h%""
		end

	mpq_div (quotient, dividend, divisor: POINTER) is
			-- Set `quotient' to `dividend/divisor'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_div_2exp (rop, op1: POINTER; op2: NATURAL_32) is
			-- Set `rop' to `op1' divided by 2 raised to `op2'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr, unsigned long) use %"gmp.h%""
		end

	mpq_neg (mnegated_operand, operand: POINTER) is
			-- Set `negated_operand' to -`operand'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_abs (rop, op: POINTER) is
			-- Set `rop' to the absolute value of `op'.
		external
			"C macro signature (mpq_ptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_inv (inverted_number, number: POINTER) is
			-- Set `inverted_number' to `1/number'.
			-- If the new denominator is zero, this
			-- routine will divide by zero.
		external
			"C macro signature (mpq_ptr, mpq_srcptr) use %"gmp.h%""
		end

feature -- Comparison

	mpq_cmp (op1, op2: POINTER): INTEGER_32 is
			-- Compare `op1' and `op2'.
			-- Return a positive value if `op1' > `op2',
			-- zero if `op1' = `op2', and a negative
			-- value if `op1' < `op2'.
		external
			"C macro signature (mpq_srcptr, mpq_srcptr): int use %"gmp.h%""
		end

	mpq_cmp_ui (op1: POINTER; num2, den2: NATURAL_32): INTEGER_32 is
			-- Compare `op1' and `num2/den2'.
			-- Return a positive value if `op1 > num2/den2',
			-- zero if `op1 = num2/den2', and a negative
			-- value if `op1 < num2/den2'.
			-- `num2' and `den2' are allowed to have common factors.
			-- Implemented as a macro and evaluates its arguments multiple times.
		external
			"C macro signature (mpq_srcptr, unsigned long, unsigned long): int use %"gmp.h%""
		end

	mpq_cmp_si (op1: POINTER; num2: INTEGER_32; den2: NATURAL_32): INTEGER_32 is
			-- Compare `op1' and `num2/den2'.
			-- Return a positive value if `op1 > num2/den2',
			-- zero if `op1 = num2/den2', and a negative
			-- value if `op1 < num2/den2'.
			-- `num2' and `den2' are allowed to have common factors.
			-- Implemented as a macro and evaluates its arguments multiple times.
		external
			"C macro signature (mpq_srcptr, long, unsigned long): int use %"gmp.h%""
		end

	mpq_sgn (op: POINTER): INTEGER_32 is
			-- Return +1 if `op' > 0, 0 if `op' = 0, and -1 if `op' < 0.
			-- This function is actually implemented as a macro.
			-- It evaluates its arguments multiple times.
		external
			"C macro signature (mpq_srcptr): int use %"gmp.h%""
		end

	mpq_equal (op1, op2: POINTER): INTEGER_32 is
			-- Return non-zero if `op1' and `op2' are equal,
			-- zero if they are non-equal.
			-- Although `mpq_cmp' can be used for the same
			-- purpose, this function is much faster.
		external
			"C macro signature (mpq_srcptr, mpq_srcptr): int use %"gmp.h%""
		end

feature -- Applying integer functions to rationals

	mpq_get_num (numerator, rational: POINTER) is
			-- Get the numerator of a rational.
		external
			"C macro signature (mpz_ptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_get_den (denominator, rational: POINTER) is
			-- Get the denominator of a rational.
		external
			"C macro signature (mpz_ptr, mpq_srcptr) use %"gmp.h%""
		end

	mpq_set_num (rational, numerator: POINTER) is
			-- Set the numerator of a rational.
		external
			"C macro signature (mpq_ptr, mpz_srcptr) use %"gmp.h%""
		end

	mpq_set_den (rational, denominator: POINTER) is
			-- Set the denominator of a rational.
		external
			"C macro signature (mpq_ptr, mpz_srcptr) use %"gmp.h%""
		end

feature -- Input and output

	mpq_out_str (stream: POINTER; base: INTEGER_32; op: POINTER): NATURAL_32 is
			-- Output `op' on stdio stream `stream', as a string of digits in base `base'.
			-- The base may vary from 2 to 36.
			-- Output is in the form `num/den' or if the denominator is 1 then just `num'.
			-- Return the number of bytes written, or if an error occurred, return 0.
		external
			"C macro signature (FILE *, int, mpq_srcptr): size_t use %"gmp.h%""
		end

	mpq_inp_str (rop, stream: POINTER; base: INTEGER_32): NATURAL_32 is
			-- Read a string of digits from `stream' and convert them to a rational in `rop'.
			-- Any initial white-space characters are read and discarded.
			-- Return the number of characters read (including white space), or 0 if a
			-- rational could not be read.
			-- The input can be a fraction like `17/63' or just an integer like `123'.
			-- Reading stops at the first character not in this form, and white space is not
			-- permitted within the string.
			-- If the input might not be in canonical form, then `mpq_canonicalize' must be
			-- called.
			-- The base can be between 2 and 36, or can be 0 in which case the leading
			-- characters of the string determine the base, `0x' or `0X' for hexadecimal,
			-- `0' for octal, or decimal otherwise.
			-- The leading characters are examined separately for the numerator and
			-- denominator of a fraction, so for instance `0x10/11' is 16/11, whereas
			-- `0x10/0x11' is 16/17.
		external
			"C macro signature (mpq_ptr, FILE *, int): size_t use %"gmp.h%""
		end

end
