indexing
	description: "Interfaces to C random number functions from The GNU MP Bignum Library"
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	RANDOM_NUMBER_FUNCTIONS

feature -- Random number

	mpz_urandomb (rop, state: POINTER; n: NATURAL_32) is
			-- Generate a uniformly distributed random integer
			-- in the range 0 to `2^n-1', inclusive.
			-- `state' must be initialized by calling one of the
			-- gmp_randinit functions (Random state initialization)
			-- before invoking this function.
		external
			"C macro signature (mpz_ptr, __gmp_randstate_struct *, unsigned long) use %"gmp.h%""
		end

	mpz_urandomm (rop, state, n: POINTER) is
			-- Generate a uniform random integer in the range
			-- 0 to `n-1', inclusive.
			-- The variable `state' must be initialized by calling
			-- one of the gmp_randinit functions (Random State Initialization)
			-- before invoking this function.
		external
			"C macro signature (mpz_ptr, __gmp_randstate_struct *, mpz_srcptr) use %"gmp.h%""
		end

	mpz_rrandomb (rop, state: POINTER; n: NATURAL_32) is
			-- Generate a random integer with long strings of zeros and ones
			-- in the binary representation.
			-- Useful for testing functions and algorithms, since this kind
			-- of random numbers have proven to be more likely to trigger
			-- corner-case bugs.
			-- The random number will be in the range 0 to `2^n-1', inclusive.
			-- The variable `state' must be initialized by calling one of the
			-- gmp_randinit functions (Random State Initialization) before
			-- invoking this function.
		external
			"C macro signature (mpz_ptr, __gmp_randstate_struct *, unsigned long) use %"gmp.h%""
		end

	mpf_urandomb (rop, state: POINTER; nbits: NATURAL_32) is
			-- Generate a uniformly distributed random float
			-- in `rop', such that 0 <= `rop' < 1, with `nbits'
			-- significant bits in the mantissa.
			-- The variable `state' must be initialized by
			-- calling one of the gmp_randinit functions
			-- (see RANDOM_STATE_FUNCTIONS) before invoking
			-- this function.
		external
			"C macro signature (mpf_ptr, __gmp_randstate_struct *, unsigned long) use %"gmp.h%""
		end

	mpf_random2 (rop: POINTER; max_size, exp: INTEGER_32) is
			-- Generate a random float of at most `max_size' limbs, with
			-- long strings of zeros and ones in the binary representation.
			-- The exponent of the number is in the interval -`exp' to
			-- `exp' (in limbs).
			-- This function is useful for testing functions and algorithms,
			-- since these kind of random numbers have proven to be more
			-- likely to trigger corner-case bugs.
			-- Negative random numbers are generated when `max_size' is
			-- negative.
		external
			"C macro signature (mpf_ptr, mp_size_t, mp_exp_t) use %"gmp.h%""
		end

	mpfr_urandomb (rop, state: POINTER): INTEGER_32 is
			-- Generate a uniformly distributed random float in
			-- the interval 0 <= rop < 1.
			-- Return 0, unless the exponent is not in the current
			-- exponent range, in which case `rop' is set to NaN
			-- and a non-zero value is returned.
		external
			"C macro signature (mpfr_ptr, __gmp_randstate_struct *): int use %"mpfr.h%""
		end

	mpfr_random2 (rop: POINTER; size, exp: INTEGER_32) is
			-- Generate a random float of at most `size'
			-- limbs, with long strings of zeros and
			-- ones in the binary representation.
			-- The exponent of the number is in the interval
			-- exp to exp.
			-- This function is useful for testing functions
			-- and algorithms, since this kind of random
			-- numbers have proven to be more likely to
			-- trigger corner-case bugs.
			-- Negative random numbers are generated when
			-- `size' is negative.
			-- Put +0 in `rop' when size if zero.
		external
			"C macro signature (mpfr_ptr, mp_size_t, mp_exp_t) use %"mpfr.h%""
		end

end
