indexing
	description: "Interfaces to C random state functions from The GNU MP Bignum Library"
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	RANDOM_STATE_FUNCTIONS

feature -- Random state initialization

	gmp_randinit_default (state: POINTER) is
			-- Initialize `state' with a default algorithm.
			-- This will be a compromise between speed and randomness,
			-- and is recommended for applications with no special
			-- requirements.
			-- Currently this is `gmp_randinit_mt'.
		external
			"C macro signature (__gmp_randstate_struct *) use %"gmp.h%""
		end

	gmp_randinit_mt (state: POINTER) is
			-- Initialize `state' for a Mersenne Twister algorithm.
			-- This algorithm is fast and has good randomness properties.
		external
			"C macro signature (__gmp_randstate_struct *) use %"gmp.h%""
		end

	gmp_randinit_lc_2exp (state, a: POINTER; c, m2exp: NATURAL_32) is
			-- Initialize state with a linear congruential algorithm `X = (a*X + c) mod 2^m2exp'.
			-- The low bits of X in this algorithm are not very random.
			-- The least significant bit will have a period no more than 2, and the second bit no
			-- more than 4, etc.
			-- For this reason only the high half of each X is actually used.
			-- When a random number of more than `m2exp/2' bits is to be generated, multiple
			-- iterations of the recurrence are used and the results concatenated.
		external
			"C macro signature (__gmp_randstate_struct *, mpz_srcptr, unsigned long, unsigned long) use %"gmp.h%""
		end

	gmp_randinit_lc_2exp_size (state: POINTER; size: NATURAL_32): INTEGER_32 is
			-- Initialize `state' for a linear congruential algorithm as per `gmp_randinit_lc_2exp'.
			-- `a', `c' and `m2exp' are selected from a table, chosen so that `size' bits (or more) of
			-- each X will be used, ie. `m2exp/2 >= size'.
			-- If successful the return value is non-zero.
			-- If `size' is bigger than the table data provides then the return value is zero.
			-- The maximum size currently supported is 128.
		external
			"C macro signature (__gmp_randstate_struct *, unsigned long): int use %"gmp.h%""
		end

	gmp_randinit_set (rop, op: POINTER) is
			-- Initialize `rop' with a copy of the algorithm and state from `op'.
		external
			"C macro signature (__gmp_randstate_struct *, __gmp_randstate_struct *) use %"gmp.h%""
		end

	gmp_randclear (state: POINTER) is
			-- Free all memory occupied by `state'.
		external
			"C macro signature (__gmp_randstate_struct *) use %"gmp.h%""
		end

feature -- Random State Seeding

	gmp_randseed (state, seed: POINTER) is
			-- Set an initial seed value into `state'.
			-- The size of a seed determines how many different sequences
			-- of random numbers that it's possible to generate.
			-- The quality of the seed is the randomness of a given seed
			-- compared to the previous seed used, and this affects the
			-- randomness of separate number sequences.
			-- The method for choosing a seed is critical if the generated
			-- numbers are to be used for important applications, such as
			-- generating cryptographic keys.
			--
		external
			"C macro signature (__gmp_randstate_struct *, mpz_srcptr) use %"gmp.h%""
		end

	gmp_randseed_ui  (state: POINTER; seed: NATURAL_32) is
			-- Set an initial seed value into `state'.
			-- The size of a seed determines how many different sequences
			-- of random numbers that it's possible to generate.
			-- The quality of the seed is the randomness of a given seed
			-- compared to the previous seed used, and this affects the
			-- randomness of separate number sequences.
			-- The method for choosing a seed is critical if the generated
			-- numbers are to be used for important applications, such as
			-- generating cryptographic keys.
			--
		external
			"C macro signature (__gmp_randstate_struct *, unsigned long) use %"gmp.h%""
		end

feature -- Random state miscellaneous

	gmp_urandomb_ui  (state: POINTER; n: NATURAL_32): NATURAL_32 is
			-- Return a uniformly distributed random number of `n' bits,
			-- ie. in the range 0 to `2^n-1' inclusive. `n' must be less
			-- than or equal to the number of bits in an unsigned long.
		external
			"C macro signature (__gmp_randstate_struct *, unsigned long): unsigned long use %"gmp.h%""
		end

	gmp_urandomm_ui  (state: POINTER; n: NATURAL_32): NATURAL_32 is
			-- Return a uniformly distributed random number in the range 0 to `n-1', inclusive.
		external
			"C macro signature (__gmp_randstate_struct *, unsigned long): unsigned long use %"gmp.h%""
		end

end
