indexing
	description: "Pseudo random number sequence of MPF_REF numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPF_RANDOM

inherit
	COUNTABLE_SEQUENCE [MPF_REF]

	RANDOM_NUMBER_FUNCTIONS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Initialization

	make (prec: NATURAL_32) is
			-- Initialize `Current'.
			-- Numbers generated will be in the
			-- interval [0, 1) with precision `prec'.
		require
			prec_positive: prec > 0
		do
			create initial_state.make_mersenne_twister
			last_i := 1
			precision := prec
			create last_i_th
			mpf_urandomb (last_i_th.item, initial_state.item, precision)
			compare_objects
		ensure
			object_comparison: object_comparison
		end

feature -- Access

	i_th (i: INTEGER_32): like item is
			-- The `i'-th random number
		local
			j: INTEGER_32
		do
			if i < last_i then
				create initial_state.make_mersenne_twister
				from
					j := 0
				until
					j = i
				loop
					mpf_urandomb (last_i_th.item, initial_state.item, precision)
					j := j + 1
				end
				last_i := j
			elseif i > last_i then
				from
					j := last_i
				until
					j = i
				loop
					mpf_urandomb (last_i_th.item, initial_state.item, precision)
					j := j + 1
				end
				last_i := j
			end
			Result := last_i_th.twin
		end

	precision: NATURAL_32

	state: RANDSTATE_STRUCT is
			--
		do
			Result := initial_state.twin
		end

feature {NONE} -- Implementation

	initial_state: RANDSTATE_STRUCT

	last_i: INTEGER_32

	last_i_th: like item

invariant
	invariant_clause: True -- Your invariant here

end
