indexing
	description: "Pseudo random number sequence of MPZ_REF numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPZ_RANDOM

inherit
	COUNTABLE_SEQUENCE [MPZ_REF]

	RANDOM_NUMBER_FUNCTIONS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Initialization

	make (n: NATURAL_32) is
			-- Initialize `Current'.
			-- Numbers generated will be between
			-- 0 and ((2 ^ `n') - 1)
		require
			n_positive: n > 0
		do
			create initial_state.make_mersenne_twister
			last_i := 1
			max := n
			create last_i_th
			mpz_urandomb (last_i_th.item, initial_state.item, max)
			compare_objects
		ensure
			object_comparison: object_comparison
		end

feature -- Access

	i_th (i: INTEGER_32): like item is
			-- The `i'-th random number
		local
			j: INTEGER_32
		do
			if i < last_i then
				create initial_state.make_mersenne_twister
				from
					j := 0
				until
					j = i
				loop
					mpz_urandomb (last_i_th.item, initial_state.item, max)
					j := j + 1
				end
				last_i := j
			elseif i > last_i then
				from
					j := last_i
				until
					j = i
				loop
					mpz_urandomb (last_i_th.item, initial_state.item, max)
					j := j + 1
				end
				last_i := j
			end
			Result := last_i_th.twin
		end

	state: RANDSTATE_STRUCT is
			--
		do
			Result := initial_state.twin
		end

feature {NONE} -- Implementation

	initial_state: RANDSTATE_STRUCT

	last_i: INTEGER_32

	last_i_th: like item

	max: NATURAL_32

invariant
	invariant_clause: True -- Your invariant here

end
