indexing
	description: "C struct used to initialize random number sequence state."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	RANDSTATE_STRUCT

inherit
	MEMORY_STRUCTURE
		redefine
			copy,
			default_create
		end

	MPZ_FUNCTIONS
		export
			{NONE} all
		redefine
			copy,
			default_create
		end

	RANDOM_STATE_FUNCTIONS
		export
			{NONE} all
		redefine
			copy,
			default_create
		end

create
	default_create,
	make_by_pointer,
--	make_linear_congruential,
	make_linear_congruential_size,
	make_mersenne_twister

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		do
			make
			gmp_randinit_default (item)
		end

--	make_linear_congruential (a: MPZ_REF; c, m2exp: NATURAL_32) is
--			-- Initialize with a linear congruential algorithm
--			-- `X = (a*X + c) mod 2^m2exp'.
--		do
--			make
--			gmp_randinit_lc_2exp (item, a.item, c, m2exp)
--		end

	make_linear_congruential_size (size: NATURAL_32) is
			-- Initialize for a linear congruential algorithm as per `gmp_randinit_lc_2exp'.
			-- `a', `c' and `m2exp' are selected from a table, chosen so that `size' bits (or more) of
			-- each X will be used, ie. `m2exp/2 >= size'.
		local
			tmp: INTEGER_32
		do
			make
			tmp := gmp_randinit_lc_2exp_size (item, size)
		end

	make_mersenne_twister is
			-- Initialize for a Mersenne Twister algorithm.
		do
			make
			gmp_randinit_mt (item)
		end

feature -- Access

	mp_lc: POINTER is
			-- Pointer to function pointers structure.
		do
			Result := mp_algdata_mp_lc (item)
		end

	seed: POINTER is
			--
		do
			Result := mp_seed (item)
		end

feature -- Measurement

	frozen structure_size: INTEGER is
			-- Size to allocate (in bytes)
		external
			"C [macro %"gmp.h%"]"
		alias
			"sizeof (__gmp_randstate_struct)"
		end

feature -- Duplication

	copy (other: like Current) is
			-- Update current object using fields of object attached
			-- to `other', so as to yield equal objects.
		do
			if managed_pointer = Void then
				default_create
			end
			gmp_randinit_set (item, other.item)
		end

feature {NONE} -- Externals

	mp_seed (p: POINTER): POINTER is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): mpz_t"
		alias
			"_mp_seed"
		end

	mp_seed_alloc (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): int"
		alias
			"_mp_seed->_mp_alloc"
		end

	mp_seed_d (p: POINTER): TYPED_POINTER [NATURAL_32] is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): mp_limb_t *"
		alias
			"_mp_seed->_mp_d"
		end

	mp_seed_size (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): int"
		alias
			"_mp_seed->_mp_size"
		end

	mp_alg (p: POINTER): GMP_RANDALG_T is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): gmp_randalg_t"
		alias
			"_mp_alg"
		end

	mp_algdata_mp_lc (p: POINTER): POINTER is
		external
			"C [struct %"gmp.h%"] (__gmp_randstate_struct): void *"
		alias
			"_mp_algdata._mp_lc"
		end

end
