indexing
	description: "C struct used to represent floating-point numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPF_STRUCT

inherit
	MEMORY
		export
			{NONE} all
		redefine
			default_create,
			dispose
		end

	MEMORY_STRUCTURE
		export
			{NONE} all
		redefine
			default_create
		end

	MPF_FUNCTIONS
		export
			{NONE} all
		redefine
			default_create
		end

create
	default_create, make_by_pointer, make_no_init

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		do
			make
			mpf_init (item)
		end

	make_no_init is
			-- Initialize `Current' without
			-- calling `mpf_init'.
		do
			make
		end

feature -- Access

	data: NATURAL_32 is
			-- Pointer to the limbs.
		do
			Result := mp_d (item)
		end

	exponent: INTEGER_32 is
			-- Exponent, in the base of `mp_limb_t'.
		do
			Result := mp_exp (item)
		end

	maximum_precision: INTEGER_32 is
			-- Max precision, in number of `mp_limb_t's.
			-- Set by `mpf_init' and modified by `mpf_set_prec'.
			-- The area pointed to by `data' contains
			-- `maximum_precision' + 1 limbs.
		do
			Result := mp_prec (item)
		end

	size: INTEGER_32 is
			-- `size.abs' is the number of limbs `data' points to.
			-- If `size' is negative this is a negative number.
		do
			Result := mp_size (item)
		end

feature -- Measurement

	frozen structure_size: INTEGER is
			-- Size to allocate (in bytes)
		external
			"C [macro %"gmp.h%"]"
		alias
			"sizeof (__mpf_struct)"
		end

feature -- Removal

	dispose is
			-- Action to be executed just before garbage collection
			-- reclaims an object.
			-- Default version does nothing; redefine in descendants
			-- to perform specific dispose actions. Those actions
			-- should only take care of freeing external resources;
			-- they should not perform remote calls on other objects
			-- since these may also be dead and reclaimed.
		do
			mpf_clear (item)
		end

feature {NONE} -- Externals

	mp_prec (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpf_struct): int"
		alias
			"_mp_prec"
		end

	mp_size (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpf_struct): int"
		alias
			"_mp_size"
		end

	mp_exp (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpf_struct): mp_exp_t"
		alias
			"_mp_exp"
		end

	mp_d (p: POINTER): NATURAL_32 is
		external
			"C [struct %"gmp.h%"] (__mpz_struct): mp_limb_t *"
		alias
			"_mp_d"
		end

end
