indexing
	description: "C struct used to represent rational numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPQ_STRUCT

inherit
	MEMORY
		export
			{NONE} all
		redefine
			default_create,
			dispose
		end

	MEMORY_STRUCTURE
		export
			{NONE} all
		redefine
			default_create
		end

	MPQ_FUNCTIONS
		export
			{NONE} all
		redefine
			default_create
		end

create
	default_create, make_by_pointer

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		do
			make
			mpq_init (item)
		end

feature -- Access

	denominator_alloc: INTEGER_32 is
			-- Number of *limbs* allocated and
			-- pointed to by `data'.
		do
			Result := mp_den_mp_alloc (item)
		end

	denominator_data: NATURAL_32 is
			-- Pointer to the limbs.
		do
			Result := mp_den_mp_d (item)
		end

	denominator_size: INTEGER_32 is
			-- `size.abs' is the number of limbs `data' points to.
			-- If `size' is negative this is a negative number.
		do
			Result := mp_den_mp_size (item)
		end

	numerator_alloc: INTEGER_32 is
			-- Number of *limbs* allocated and
			-- pointed to by `data'.
		do
			Result := mp_num_mp_alloc (item)
		end

	numerator_data: NATURAL_32 is
			-- Pointer to the limbs.
		do
			Result := mp_num_mp_d (item)
		end

	numerator_size: INTEGER_32 is
			-- `size.abs' is the number of limbs `data' points to.
			-- If `size' is negative this is a negative number.
		do
			Result := mp_num_mp_size (item)
		end

feature -- Measurement

	frozen structure_size: INTEGER is
			-- Size to allocate (in bytes)
		external
			"C [macro %"gmp.h%"]"
		alias
			"sizeof (__mpq_struct)"
		end

feature -- Removal

	dispose is
			-- Action to be executed just before garbage collection
			-- reclaims an object.
			-- Default version does nothing; redefine in descendants
			-- to perform specific dispose actions. Those actions
			-- should only take care of freeing external resources;
			-- they should not perform remote calls on other objects
			-- since these may also be dead and reclaimed.
		do
			mpq_clear (item)
		end

feature {NONE} -- Externals

	mp_num_mp_alloc (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpq_struct): int"
		alias
			"_mp_num._mp_alloc"
		end

	mp_num_mp_size (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpq_struct): int"
		alias
			"_mp_num._mp_size"
		end

	mp_num_mp_d (p: POINTER): NATURAL_32 is
		external
			"C [struct %"gmp.h%"] (__mpq_struct): mp_limb_t *"
		alias
			"_mp_num._mp_d"
		end

	mp_den_mp_alloc (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpq_struct): int"
		alias
			"_mp_den._mp_alloc"
		end

	mp_den_mp_size (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpq_struct): int"
		alias
			"_mp_den._mp_size"
		end

	mp_den_mp_d (p: POINTER): NATURAL_32 is
		external
			"C [struct %"gmp.h%"] (__mpq_struct): mp_limb_t *"
		alias
			"_mp_den._mp_d"
		end

end
