indexing
	description: "C struct used to represent integer numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPZ_STRUCT

inherit
	MEMORY
		export
			{NONE} all
		redefine
			default_create,
			dispose
		end

	MEMORY_STRUCTURE
		rename
			make as memory_structure_make
		export
			{NONE} all
		redefine
			default_create
		end

	MPZ_FUNCTIONS
		export
			{NONE} all
		redefine
			default_create
		end

create
	default_create, make_by_pointer

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		do
			memory_structure_make
			mpz_init (item)
		end

feature -- Access

	alloc: INTEGER_32 is
			-- Number of *limbs* allocated and
			-- pointed to by `data'.
		do
			Result := mp_alloc (item)
		end

	data: NATURAL_32 is
			-- Pointer to the limbs.
		do
			Result := mp_d (item)
		end

	size: INTEGER_32 is
			-- `size.abs' is the number of limbs `data' points to.
			-- If `size' is negative this is a negative number.
		do
			Result := mp_size (item)
		end

feature -- Measurement

	frozen structure_size: INTEGER is
			-- Size to allocate (in bytes)
		external
			"C [macro %"gmp.h%"]"
		alias
			"sizeof (__mpz_struct)"
		end

feature -- Removal

	dispose is
			-- Action to be executed just before garbage collection
			-- reclaims an object.
			-- Default version does nothing; redefine in descendants
			-- to perform specific dispose actions. Those actions
			-- should only take care of freeing external resources;
			-- they should not perform remote calls on other objects
			-- since these may also be dead and reclaimed.
		do
			mpz_clear (item)
		end

feature {NONE} -- Externals

	mp_alloc (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpz_struct): int"
		alias
			"_mp_alloc"
		end

	mp_size (p: POINTER): INTEGER_32 is
		external
			"C [struct %"gmp.h%"] (__mpz_struct): int"
		alias
			"_mp_size"
		end

	mp_d (p: POINTER): NATURAL_32 is
		external
			"C [struct %"gmp.h%"] (__mpz_struct): mp_limb_t *"
		alias
			"_mp_d"
		end

end
