indexing
	description: "Complex number with MPFR_REF real and imaginary parts."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPFR_COMPLEX_REF

inherit
	MPFR_MATH
		export
			{NONE} all
		undefine
			copy,
			default_create,
			is_equal,
			out
		end

	MP_COMPLEX [MPFR_REF]

create
	default_create,
	make,
	make_from_integer_32,
	make_from_real_64,
	make_real,
	make_precision,
	make_precision_2,
	make_precision_nan,
	make_precision_nan_2

convert
	make_from_integer_32 ({INTEGER_32}),
	make_from_real_64 ({REAL_64}),
	make_real ({MPFR_REF}),
	to_real: {MPFR_REF}

feature {NONE} -- Initialization

	make_from_integer_32 (v: INTEGER_32) is
			-- Initialize `Current' from `v'.
		do
			real := v
			imaginary := 0.0
		end

	make_from_real_64 (v: REAL_64) is
			-- Initialize `Current' from `v'.
		do
			real := v
			imaginary := 0.0
		end

	make_precision (prec: NATURAL_32; rnd: MPFR_RND_T) is
			-- Initialize with the precision of `real' and
			-- `imaginary' set to `prec'.
		do
			create real.make_precision (prec, rnd)
			create imaginary.make_precision (prec, rnd)
		end

	make_precision_2 (re, im: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T) is
			-- Initialize with the precision of `real'
			-- set to `re' and the precision of `imaginary'
			-- set to `im'.
		do
			create real.make_precision (re, re_rnd)
			create imaginary.make_precision (im, im_rnd)
		end

	make_precision_nan (prec: NATURAL_32; rnd: MPFR_RND_T) is
			-- Initialize with the precision of `real' and
			-- `imaginary' set to `prec'.
			-- `real' and `imaginary' will be NaN.
		do
			create real.make_precision_nan (prec,  rnd)
			create imaginary.make_precision_nan (prec, rnd)
		end

	make_precision_nan_2 (re, im: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T) is
			-- Initialize with the precision of `real'
			-- set to `re' and the precision of `imaginary'
			-- set to `im'.
			-- `real' and `imaginary' will be NaN.
		do
			create real.make_precision_nan (re, re_rnd)
			create imaginary.make_precision_nan (im, im_rnd)
		end

feature -- Element change

	set_precision (prec: NATURAL_32) is
			-- Set the precision of `real' and `imaginary' to `prec'.
		do
			real.set_precision (prec)
			imaginary.set_precision (prec)
		end

	set_precision_2 (re, im: NATURAL_32) is
			-- Set the precision of `real' to `re' and
			-- `imaginary' to `im'.
		do
			real.set_precision (re)
			imaginary.set_precision (im)
		end

	set_rounding_mode (rnd: MPFR_RND_T) is
			-- Set the rounding mode of `real' and `imaginary' to `rnd'.
		do
			real.set_rounding_mode (rnd)
			imaginary.set_rounding_mode (rnd)
		end

	set_rounding_mode_2 (re, im: MPFR_RND_T) is
			-- Set the rounding mode of `real' and `imaginary'
			-- to `re' and `im' respectively.
		do
			real.set_rounding_mode (re)
			imaginary.set_rounding_mode (im)
		end

feature -- Basic operations

	abs: like real is
			-- Absolute value.
		do
			Result := sqrt (real * real + imaginary * imaginary, real.precision.max (imaginary.precision), real.rounding_mode)
		end

	arg: like real is
			-- Counter-clockwise angle from positive x-axis in radians
			-- (0.0, 0.0) = 0.0
		do
			Result := atan2 (imaginary, real, real.precision.max (imaginary.precision), real.rounding_mode)
		end

end
