indexing
	description: "Multiple precision floating-point numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPFR_REF

inherit
	MP_NUMBER
		redefine
			infix "<=",
			infix ">",
			infix ">="
		end

	MPFR_STRUCT
		rename
			sign as mpfr_struct_sign
		export
			{NONE} all
			{MPFR_REF, MPFR_COMPLEX_MATH, MPFR_MATH, MPFR_RANDOM} item
		redefine
			copy,
			is_equal,
			out
		end

create
	default_create,
	make_from_double,
	make_from_integer,
	make_from_integer_32,
	make_from_mpf_ref,
	make_from_mpq_ref,
	make_from_mpz_ref,
	make_from_natural_32,
	make_from_real_64,
	make_from_string,
	make_precision,
	make_precision_nan

convert
	make_from_double ({REAL_64}),
	make_from_integer ({INTEGER_32}),
	make_from_mpf_ref ({MPF_REF}),
	make_from_mpq_ref ({MPQ_REF}),
	make_from_mpz_ref ({MPZ_REF}),
	make_from_string ({STRING_8}),
	to_real_64: {REAL_64}

feature {NONE} -- Initialization

	make_from_double (v: REAL_64) is
			-- Initialize `Current' from `v'.
		do
			make
			mpfr_init (item)
			rounding_mode := Default_rounding_mode
			last_result := mpfr_set_str (item,
			                             (create {C_STRING}.make (v.out)).item,
			                             {BASE_CONSTANTS}.Decimal_base,
			                             rounding_mode.item)
		ensure
			rounding_mode_set: rounding_mode = Default_rounding_mode
		end

	make_from_integer (v: INTEGER_32) is
			-- Initialize `Current' from `v'.
		do
			make
			mpfr_init (item)
			rounding_mode := Default_rounding_mode
			last_result := mpfr_set_si (item, v, rounding_mode.item)
		ensure
			is_integer: is_integer
			rounding_mode_set: rounding_mode = Default_rounding_mode
		end

	make_from_integer_32 (v: INTEGER_32; prec: NATURAL_32; rnd: MPFR_RND_T) is
			-- Initialize `Current' from `v'.
		do
			make
			mpfr_init2 (item, prec)
			rounding_mode := rnd
			last_result := mpfr_set_si (item, v, rnd)
		ensure
			is_integer: is_integer
			precision_set: precision = prec
			rounding_mode_set: rounding_mode = rnd
		end

	make_from_mpf_ref (v: MPF_REF) is
			-- Initialize `Current' from `v'.
		do
			make
			mpfr_init2 (item, v.precision)
			rounding_mode := Default_rounding_mode
			last_result := mpfr_set_f (item, v.item, Default_rounding_mode)
		ensure
			precision_set: precision = v.precision
			rounding_mode_set: rounding_mode = Default_rounding_mode
		end

	make_from_mpq_ref (v: MPQ_REF) is
			-- Initialize `Current' from `v'.
		do
			make
			mpfr_init2 (item, v.precision)
			rounding_mode := Default_rounding_mode
			last_result := mpfr_set_q (item, v.item, Default_rounding_mode)
		ensure
			precision_set: precision = v.precision
			rounding_mode_set: rounding_mode = Default_rounding_mode
		end

	make_from_mpz_ref (v: MPZ_REF) is
			-- Initialize `Current' from `v'.
		do
			make
			mpfr_init2 (item, v.precision)
			rounding_mode := Default_rounding_mode
			last_result := mpfr_set_z (item, v.item, Default_rounding_mode)
		ensure
			is_integer: is_integer
			precision_set: precision = v.precision
			rounding_mode_set: rounding_mode = Default_rounding_mode
		end

	make_from_natural_32 (v, prec: NATURAL_32; rnd: MPFR_RND_T) is
			-- Initialize `Current' from `v'.
		do
			make
			mpfr_init2 (item, prec)
			rounding_mode := rnd
			last_result := mpfr_set_ui (item, v, rnd)
		ensure
			is_integer: is_integer
			precision_set: precision = prec
			rounding_mode_set: rounding_mode = rnd
		end

	make_from_real_64 (v: REAL_64; prec: NATURAL_32; rnd: MPFR_RND_T) is
			-- Initialize `Current' from `v'.
		do
			make
			mpfr_init2 (item, prec)
			rounding_mode := rnd
			last_result := mpfr_set_str (item,
			                             (create {C_STRING}.make (v.out)).item,
			                             {BASE_CONSTANTS}.Decimal_base,
			                             rnd)
		ensure
			precision_set: precision = prec
			rounding_mode_set: rounding_mode = rnd
		end

	make_from_string (v: STRING_8) is
			-- Initialize `Current' from `v' from `v'.
		require
			v_not_void: v /= Void
		do
			make
			mpfr_init2 (item, (v.count * 8).to_natural_32)
			rounding_mode := Default_rounding_mode
			last_result := mpfr_set_str (item,
			                             (create {C_STRING}.make (v)).item,
			                             {BASE_CONSTANTS}.Decimal_base,
			                             rounding_mode.item)
		ensure
			precision_set: precision = (v.count * 8).to_natural_32
			rounding_mode_set: rounding_mode = Default_rounding_mode
		end

	make_precision (prec: NATURAL_32; rnd: MPFR_RND_T) is
			-- Initialize `Current' to 0 with
			-- `precision' set to `prec'.
		require
			prec_valid: is_valid_precision (prec)
		do
			make
			mpfr_init2 (item, prec)
			rounding_mode := rnd
			last_result := mpfr_set_si (item, 0, rnd)
		ensure
			precision_set: precision = prec
			rounding_mode_set: rounding_mode = rnd
			is_zero: is_zero
		end

	make_precision_nan (prec: NATURAL_32; rnd: MPFR_RND_T) is
			-- Initialize `Current'.
		require
			prec_valid: is_valid_precision (prec)
		do
			make
			mpfr_init2 (item, prec)
			rounding_mode := rnd
		ensure
			precision_set: precision = prec
			rounding_mode_set: rounding_mode = rnd
			is_nan: is_nan
		end

feature -- Access

	last_result: INTEGER_32
			-- Last result of a call to
			-- an MPFR function.

	one: like Current is
			-- Neutral element for "*" and "/"
		do
			create Result.make_from_real_64 (1.0, precision, rounding_mode)
		end

	rounding_mode: MPFR_RND_T
			-- Rounding mode.

	sign: INTEGER is
			-- Sign value (0, -1 or 1)
		local
			sgn: INTEGER_32
		do
			sgn := mpfr_sgn (item)
			if sgn > 0 then
				Result := 1
			elseif sgn < 0 then
				Result := -1
			end
		ensure
			three_way: Result = three_way_comparison (zero)
		end

	zero: like Current is
			-- Neutral element for "+" and "-"
		do
			create Result.make_from_real_64 (0.0, precision, rounding_mode)
		end

feature -- Measurement

	precision: NATURAL_32 is
			-- Number of bits allocated for `Current'.
		do
			Result := mpfr_get_prec (item)
		end

feature -- Status report

	Default_rounding_mode: MPFR_RND_T is
			-- Default rounding mode.
		once
			create Result.make (create {MPFR_RND_T}.make ({MPFR_RND_T}.Gmp_rndn))
		end

	divisible (other: like Current): BOOLEAN is
			-- May current object be divided by `other'?
			-- NaN is a possible result of `infix "/"'.
		do
			Result := True
		end

	exponentiable (other: NUMERIC): BOOLEAN is
			-- May current object be elevated to the power `other'?
			-- NaN is a possible result of `infix "^"'.
		do
			Result := True
		end

	fits_integer_16: BOOLEAN is
			-- Does `Current' fit in a INTEGER_16
			-- when rounded to an integer using
			-- `rounding_mode'?
		do
			Result := mpfr_fits_sshort_p (item, rounding_mode) /= 0
		end

	fits_integer_32: BOOLEAN is
			-- Does `Current' fit in a INTEGER_32
			-- when rounded to an integer using
			-- `rounding_mode'?
		do
			Result := mpfr_fits_slong_p (item, rounding_mode) /= 0
		end

	fits_natural_16: BOOLEAN is
			-- Does `Current' fit in a NATURAL_16
			-- when rounded to an integer using
			-- `rounding_mode'?
		do
			Result := mpfr_fits_ushort_p (item, rounding_mode) /= 0
		end

	fits_natural_32: BOOLEAN is
			-- Does `Current' fit in a NATURAL_32
			-- when rounded to an integer using
			-- `rounding_mode'?
		do
			Result := mpfr_fits_ulong_p (item, rounding_mode) /= 0
		end

	is_infinite: BOOLEAN is
			-- Is `Current' an infinity?
		do
			Result := mpfr_inf_p (item) /= 0
		end

	is_integer: BOOLEAN is
			-- Is `Current' an integer (no fractional part)?
		do
			Result := mpfr_integer_p (item) /= 0
		end

	is_nan: BOOLEAN is
			-- Is `Current' Nan?
		do
			Result := mpfr_nan_p (item) /= 0
		end

	is_unordered: BOOLEAN is
			-- Is `Current' or `other' NaN?
		do
			Result := mpfr_number_p (item) /= 0
		end

	is_ordinary: BOOLEAN is
			-- Is `Current' an ordinary number?
			-- (Not infinity or NaN)
		do
			Result := mpfr_number_p (item) /= 0
		end

	is_valid_precision (prec: NATURAL_32): BOOLEAN is
			-- Is `prec' a valid value for `precision'?
		do
			Result := ({MPFR_PRECISION_LIMITS}.Mpfr_prec_min < prec) and then
			           (prec <= {MPFR_PRECISION_LIMITS}.Mpfr_prec_max)
		end

	is_valid_rounding_mode (rnd: INTEGER_32): BOOLEAN is
			-- Is `rnd' a valid value for `rounding_mode'?
		do
			Result := (rnd = {MPFR_RND_T}.Gmp_rndn) or else
			          (rnd = {MPFR_RND_T}.Gmp_rndd) or else
			          (rnd = {MPFR_RND_T}.Gmp_rndu) or else
			          (rnd = {MPFR_RND_T}.Gmp_rndz)
		end

	is_zero: BOOLEAN is
			-- Is `Current' `zero'?
		do
			Result := mpfr_zero_p (item) /= 0
		end

feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is current object less than `other'?
		do
			Result := mpfr_cmp (item, other.item) < 0
		end

	infix "<=" (other: like Current): BOOLEAN is
			-- Is current object less than or equal to `other'?
		do
			Result := mpfr_lessequal_p (item, other.item) /= 0
		ensure then
			definition: Result = ((Current < other) or is_equal (other))
		end

	infix ">" (other: like Current): BOOLEAN is
			-- Is current object greater than `other'?
		do
			Result := mpfr_greater_p (item, other.item) /= 0
		ensure then
			definition: Result = (other < Current)
		end

	infix ">=" (other: like Current): BOOLEAN is
			-- Is current object greater than or equal to `other'?
		do
			Result := mpfr_greaterequal_p (item, other.item) /= 0
 		ensure then
			definition: Result = (other <= Current)
		end

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		do
			Result := mpfr_equal_p (item, other.item) /= 0
		end

feature -- Element change

	set_from_integer_32 (v: INTEGER_32) is
			-- Initialize `Current' from `v'.
		do
			last_result := mpfr_set_si (item, v, rounding_mode)
		ensure
			is_integer: is_integer
		end

	set_from_mpf_ref (v: MPF_REF) is
			-- Initialize `Current' from `v'.
		do
			last_result := mpfr_set_f (item, v.item, rounding_mode)
		end

	set_from_mpq_ref (v: MPQ_REF) is
			-- Initialize `Current' from `v'.
		do
			last_result := mpfr_set_q (item, v.item, rounding_mode)
		end

	set_from_mpz_ref (v: MPZ_REF) is
			-- Initialize `Current' from `v'.
		do
			last_result := mpfr_set_z (item, v.item, rounding_mode)
		ensure
			is_integer: is_integer
		end

	set_from_natural_32 (v: NATURAL_32) is
			-- Initialize `Current' from `v'.
		do
			last_result := mpfr_set_ui (item, v, rounding_mode)
		ensure
			is_integer: is_integer
		end

	set_from_real_64 (v: REAL_64) is
			-- Initialize `Current' from `v'.
		do
			last_result := mpfr_set_str (item,
			                             (create {C_STRING}.make (v.out)).item,
			                             {BASE_CONSTANTS}.Decimal_base,
			                             rounding_mode)
		end

	set_from_string (v: STRING_8) is
			-- Initialize `Current' from `v' from `v'.
		require
			v_not_void: v /= Void
		do
			last_result := mpfr_set_str (item,
			                             (create {C_STRING}.make (v)).item,
			                             {BASE_CONSTANTS}.Decimal_base,
			                             rounding_mode)
		end

	set_precision (prec: NATURAL_32) is
			-- Set `precision' to at least `prec'.
		require
			prec_valid: is_valid_precision (prec)
		do
			last_result := mpfr_prec_round (item, prec, rounding_mode)
		end

	set_rounding_mode (v: MPFR_RND_T) is
			-- Set `rounding_mode' to `v'.
		do
			rounding_mode := v
		ensure
			rounding_mode_set: rounding_mode = v
		end

feature -- Conversion

	to_integer_32: INTEGER_32 is
			-- Convert to an INTEGER_32.
		require
			fits_integer_32: fits_integer_32
		do
			Result := mpfr_get_si (item, rounding_mode)
		end

	to_mpf_ref: MPF_REF is
			-- Convert to an MPF_REF.
		do
			create Result.make_precision (precision)
			last_result := mpfr_get_f (Result.item, item, rounding_mode)
		end

	to_mpq_ref: MPQ_REF is
			-- Convert to an MPQ_REF.
		do
			create Result.make_from_mpf_ref (to_mpf_ref)
		end

	to_mpz_ref: MPZ_REF is
			-- Convert to an MPZ_REF.
		do
			create Result
			mpfr_get_z (Result.item, item, rounding_mode)
		end

	to_natural_32: NATURAL_32 is
			-- Convert to an NATURAL_32.
		require
			fits_natural_32: fits_natural_32
		do
			Result := mpfr_get_ui (item, rounding_mode)
		end

	to_real_64: REAL_64 is
			-- Convert to a REAL_64.
		do
			Result := mpfr_get_d (item, rounding_mode)
		end

feature -- Duplication

	copy (other: like Current) is
			-- Update current object using fields of object attached
			-- to `other', so as to yield equal objects.
		do
			if managed_pointer = Void then
				default_create
			end
			mpfr_set_prec (item, other.precision)
			last_result := mpfr_set (item, other.item, other.rounding_mode)
		end

feature -- Basic operations

	abs: like Current is
			-- Absolute value
		do
			create Result.make_precision_nan (precision, rounding_mode)
			last_result := mpfr_abs (Result.item, item, rounding_mode)
		end

	infix "+" (other: like Current): like Current is
			-- Sum with `other' (commutative).
		do
			create Result.make_precision_nan (precision.max (other.precision), rounding_mode)
			last_result := mpfr_add (Result.item, item, other.item, rounding_mode)
		end

	infix "-" (other: like Current): like Current is
			-- Result of subtracting `other'
		do
			create Result.make_precision_nan (precision.max (other.precision), rounding_mode)
			last_result := mpfr_sub (Result.item, item, other.item, rounding_mode)
		end

	infix "*" (other: like Current): like Current is
			-- Product by `other'
		do
			create Result.make_precision_nan (precision.max (other.precision), rounding_mode)
			last_result := mpfr_mul (Result.item, item, other.item, rounding_mode)
		end

	infix "/" (other: like Current): like Current is
			-- Division by `other'
		do
			create Result.make_precision_nan (precision.max (other.precision), rounding_mode)
			last_result := mpfr_div (Result.item, item, other.item, rounding_mode)
		end

	infix "^" (other: like Current): like Current is
			-- Power of Current by `other'
		require
			good_exponent: exponentiable (other)
		do
			create Result.make_precision_nan (precision.max (other.precision), rounding_mode)
			last_result := mpfr_pow (Result.item, item, other.item, rounding_mode)
		ensure
			result_not_void: Result /= Void
		end

	prefix "+": like Current is
			-- Unary plus
		do
			create Result.make_precision_nan (precision, rounding_mode)
			last_result := mpfr_set (Result.item, item, rounding_mode)
		end

	prefix "-": like Current is
			-- Unary minus
		do
			create Result.make_precision_nan (precision, rounding_mode)
			last_result := mpfr_neg (Result.item, item, rounding_mode)
		end

feature -- Output

	out: STRING is
			-- New string containing terse printable representation
			-- of current object
		local
			exp, i: INTEGER_32
			res: C_STRING
			str: STRING
			tmp1, tmp2: POINTER
		do
			if is_nan then
				Result := "NaN"
			elseif is_infinite then
				if Current < zero then
					Result := "-Inf"
				else
					Result := "Inf"
				end
			else
				tmp1 := mpfr_get_str (tmp2, $exp, {BASE_CONSTANTS}.Decimal_base, 0, item, rounding_mode)
				create res.make_by_pointer (tmp1)
				mpfr_free_str (tmp1)
				if is_equal (zero) then
					Result := "0"
				elseif is_integer then
					Result := res.string
				elseif exp = 0 then
					Result := "0." + res.string
				elseif exp < 0 then
					from
						i := -1
						str := "0."
					until
						i < exp
					loop
						str.append_character ('0')
						i := i - 1
					end
					Result := str + res.string
					Result.prune ('-')
					if Current < zero then
						Result.prepend_character ('-')
					end
				else
					if Current > zero then
						Result := res.string
						Result.insert_character ('.', exp + 1)
					else
						Result := res.string
						Result.insert_character ('.', exp + 2)
					end
				end
			end
			Result.prune_all_trailing ('0')
			if Result.is_empty then
				Result := "0"
			end
		end

invariant
	valid_precision: is_valid_precision (precision)

end
