indexing
	description: "Pseudo random number sequence of MPFR_REF numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPFR_RANDOM

inherit
	COUNTABLE_SEQUENCE [MPFR_REF]

	RANDOM_NUMBER_FUNCTIONS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Initialization

	make (prec: NATURAL_32; rnd: MPFR_RND_T) is
			-- Initialize `Current'.
			-- Numbers generated will be in the
			-- interval [0, 1) with precision `prec'.
		require
			prec_positive: prec > 0
		local
			tmp: INTEGER_32
		do
			create initial_state.make_mersenne_twister
			last_i := 1
			precision := prec
			create last_i_th.make_precision (prec, rnd)
			tmp := mpfr_urandomb (last_i_th.item, initial_state.item)
			compare_objects
		ensure
			object_comparison: object_comparison
		end

feature -- Access

	i_th (i: INTEGER_32): like item is
			-- The `i'-th random number
		local
			j, tmp: INTEGER_32
		do
			if i < last_i then
				create initial_state.make_mersenne_twister
				from
					j := 0
				until
					j = i
				loop
					tmp := mpfr_urandomb (last_i_th.item, initial_state.item)
					j := j + 1
				end
				last_i := j
			elseif i > last_i then
				from
					j := last_i
				until
					j = i
				loop
					tmp := mpfr_urandomb (last_i_th.item, initial_state.item)
					j := j + 1
				end
				last_i := j
			end
			Result := last_i_th.twin
		end

	precision: NATURAL_32
			-- Precision of the generated random number.

	state: RANDSTATE_STRUCT is
			--
		do
			Result := initial_state.twin
		end

feature {NONE} -- Implementation

	initial_state: RANDSTATE_STRUCT

	last_i: INTEGER_32

	last_i_th: like item

end
