indexing
	description: "C struct used to represent floating-point numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPFR_STRUCT

inherit
	MEMORY
		export
			{NONE} all
		redefine
			default_create,
			dispose
		end

	MEMORY_STRUCTURE
		export
			{NONE} all
		redefine
			default_create
		end

	MPFR_FUNCTIONS
		export
			{NONE} all
		redefine
			default_create
		end

create
	default_create, make_by_pointer

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		local
			tmp: INTEGER_32
		do
			make
			mpfr_init (item)
			tmp := mpfr_set_si (item, 0, {MPFR_RND_T}.Gmp_rndn)
		end

feature -- Access

	data: TYPED_POINTER [NATURAL_32] is
			-- Pointer to the limbs.
		do
			Result := mpfr_d (item)
		end

	exponent: INTEGER_32 is
			-- Exponent, in the base of `mp_limb_t'.
		do
			Result := c_mpfr_exp (item)
		end

	maximum_precision: NATURAL_32 is
			-- Max precision, in number of `mp_limb_t's.
		do
			Result := mpfr_prec (item)
		end

	sign: INTEGER_32 is
			-- Sign.
		do
			Result := mpfr_sign (item)
		end

feature -- Measurement

	frozen structure_size: INTEGER is
			-- Size to allocate (in bytes)
		external
			"C [macro %"mpfr.h%"]"
		alias
			"sizeof (__mpfr_struct)"
		end

feature -- Removal

	dispose is
			-- Action to be executed just before garbage collection
			-- reclaims an object.
			-- Default version does nothing; redefine in descendants
			-- to perform specific dispose actions. Those actions
			-- should only take care of freeing external resources;
			-- they should not perform remote calls on other objects
			-- since these may also be dead and reclaimed.
		do
			mpfr_clear (item)
		end

feature {NONE} -- Externals

	mpfr_prec (p: POINTER): NATURAL_32 is
		external
			"C [struct %"mpfr.h%"] (__mpfr_struct): mpfr_prec_t"
		alias
			"_mpfr_prec"
		end

	mpfr_sign (p: POINTER): INTEGER_32 is
		external
			"C [struct %"mpfr.h%"] (__mpfr_struct): mpfr_sign_t"
		alias
			"_mpfr_sign"
		end

	c_mpfr_exp (p: POINTER): INTEGER_32 is
		external
			"C [struct %"mpfr.h%"] (__mpfr_struct): mp_exp_t"
		alias
			"_mpfr_exp"
		end

	mpfr_d (p: POINTER): TYPED_POINTER [NATURAL_32] is
		external
			"C [struct %"mpfr.h%"] (__mpfr_struct): mp_limb_t *"
		alias
			"_mpfr_d"
		end

end
